<?php

require_once 'm2.constants.inc';

/**
 * m2_admin_security_page
 * m2_admin_security_form
 */

function m2_admin_security_page() {
  drupal_set_title(t('Administer Menu2'));
  $output = m2_form_render('m2_admin_security_form');
  return theme('m2_content_wrapper', array('class' => array('m2-tab', 'm2-security-tab'), 'markup' => $output));
}


function m2_admin_security_form($form, &$form_state) {
  $form['perms'] = array(
    '#type' => 'fieldset',
    '#title' => t('Users and Permissions'),
  );
  foreach (m2_permission() as $c_perm_id => $c_perm_info) {
    $header = array(
      array('class' => array('users'), 'data' => t('Users')),
      array('class' => array('roles'), 'data' => t('through Role')),
    );
    $rows = array();
    $roles_by_perm = _m2_sec_get_roles_by_permissions(array($c_perm_id));
 /* check if anonymous or authenticated role have permission */
    if (isset($roles_by_perm[DRUPAL_ANONYMOUS_RID]))     $rows[] = array('class' => array('error'),   'data' => array(array('colspan' => 2, 'data' => t('All anonymous users have this permission!'))));
    if (isset($roles_by_perm[DRUPAL_AUTHENTICATED_RID])) $rows[] = array('class' => array('warning'), 'data' => array(array('colspan' => 2, 'data' => t('All authenticated users have this permission!'))));
 /* collect info about user roles by current permission */
    $users_stats = array();
    foreach ($roles_by_perm as $c_role_id => $c_role_name) {
      $users_by_role = _m2_sec_get_users_by_roles(array($c_role_id));
      foreach ($users_by_role as $c_user_id => $c_user_name) {
        if (!isset($users_stats[$c_user_id])) $users_stats[$c_user_id] = array('name' => $c_user_name, 'roles' => array());
        $users_stats[$c_user_id]['roles'][] = $c_role_name;
      }
    }
    foreach ($users_stats as $user_id => $user_info) {
      $rows[] = array('data' => array(
        array('class' => array('users'), 'data' => user_access('access user profiles') ? l($user_info['name'], "user/$user_id", array('attributes' => array('target' => '_blank'))) : $user_info['name']),
        array('class' => array('roles'), 'data' => implode(', ', $user_info['roles'])),
      ));
    }
 /* report */
    $form['perms'][$c_perm_id]['title'] = array(
      '#markup' => theme('m2_title', array('type' => 'h4', 'markup' => t('Permission <strong>"%perm_name"</strong>', array('%perm_name' => $c_perm_info['title'])))),
    );
    $form['perms'][$c_perm_id]['info'] = array(
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t('No Users to access to this Permission across any Role.'),
    );
  }
  return $form;
}


/**
 * _m2_sec_get_roles_by_permissions | return array(N) or array()
 * _m2_sec_get_users_by_roles       | return array(N) or array()
 */

function _m2_sec_get_roles_by_permissions($perms_id) {
  if (is_array($perms_id) && count($perms_id)) {
    $query = db_select('role_permission', 'p');
    $query->join('role', 'r', 'p.rid = r.rid');
    $query->fields('p', array('rid'))->fields('r', array('name'));
    $query->condition('p.permission', $perms_id, 'in');
    $query_result = $query->execute()->fetchAllKeyed();
    return $query_result ? $query_result : array();
  } else {
    return array();
  }
}


function _m2_sec_get_users_by_roles($roles_id) {
  if (is_array($roles_id) && count($roles_id)) {
    $query = db_select('users_roles', 'r');
    $query->join('users', 'u', 'u.uid = r.uid');
    $query->fields('r', array('uid'))->fields('u', array('name'));
    $query->condition('r.rid', $roles_id, 'in');
    $query->orderBy('name');
    $query_result = $query->execute()->fetchAllKeyed();
    return $query_result ? $query_result : array();
  } else {
    return array();
  }
}

